#include "config.h"
#ifdef BUTTON
#include "lisp.h"
#include "x11term.h"
#include "screenX.h"

#define MAX_STIPPLES 256
static Pixmap stipples[MAX_STIPPLES];
static char allocated_stipples[MAX_STIPPLES];

#define MAX_STIPPLE_BYTES 8192
#define BITS_PER_BYTE 8

static Pixmap
make_stipple(data,size)
     char *data;
     int size;
{
  int width = 0, height = 1, hsize = 0, byteno = 0, bit = 0;
  unsigned char bits [MAX_STIPPLE_BYTES];
  unsigned char byte = 0;
  Pixmap pixmap;
  BLOCK_INPUT_DECLARE();

  /* first determine the width */
  for (width = 0; (width < size && data[width] != '\n'); width++);
  while (size-- > 0) {
    if (*data == '\n')
      if (hsize != width)
	error ("irregular stipple data");
      else {
	if (byteno >= MAX_STIPPLE_BYTES)
	  error ("stipple data too large");
	if (width > BITS_PER_BYTE && bit < BITS_PER_BYTE)
	  byte <<= BITS_PER_BYTE - bit;
	bits[byteno++] = byte;
	byte = bit = hsize = 0;
	height++;
      }
    else {
      if (++bit > BITS_PER_BYTE) {
	if (byteno >= MAX_STIPPLE_BYTES)
	  error ("stipple data too large");
	bits[byteno++] = byte;
	byte = 0;
	bit = 1;
      }
      byte = (byte << 1) | (*data != ' ' & *data != '0');
      hsize++;
    }
    data++;
  }
  if (bit > 0) {
    if (hsize != width)
      error ("irregular stipple data");
    if (byteno >= MAX_STIPPLE_BYTES)
      error ("stipple data too large");
    if (width > BITS_PER_BYTE && bit < BITS_PER_BYTE)
      byte <<= BITS_PER_BYTE - bit;
    bits[byteno] = byte;
  }
  BLOCK_INPUT();
  pixmap = XCreatePixmapFromBitmapData(cur_Xscreen->display,
				       cur_Xscreen->xid,
				       bits,
				       width,
				       height,
				       BlackPixel(cur_Xscreen->display,DefaultScreen(cur_Xscreen->display)),
				       WhitePixel(cur_Xscreen->display,DefaultScreen(cur_Xscreen->display)),
				       1);
  UNBLOCK_INPUT();
  return (pixmap);
}

DEFUN("make-stipple",Fmake_stipple,Smake_stipple,1,1,0,
      "Make a stipple. argument is a string containing several lines\n\
of equal length. Put spaces or zeros where you want the\n\
background color, and anything else where you want the\n\
foreground color.")
  (data)
     Lisp_Object data;
{
  int idx;
  CHECK_STRING (data,0);
  for (idx=0;idx<MAX_STIPPLES;idx++)
    if (allocated_stipples[idx] == 0) break;
  if (idx>=MAX_STIPPLES)
    error ("too many pixmaps");
  stipples[idx] = make_stipple(XSTRING(data)->data,XSTRING(data)->size);
  allocated_stipples[idx] = 1;
  return (make_number(idx));
}

DEFUN("kill-stipple",Fkill_stipple,Skill_stipple,1,1,0,
      "kill a stipple: deallocate pixmap and release stipple id.\n\
return t if there was a stipple to deallocate, nil otherwise.")
     (stipple)
     Lisp_Object stipple;
{
  int idx;
  BLOCK_INPUT_DECLARE();
  CHECK_NUMBER (stipple,0);
  idx = XFASTINT (stipple);
  if (idx < 0 || idx >= MAX_STIPPLES)
    error ("stipple id out of range: %u",idx);
  if (allocated_stipples[idx] != 0) {
    BLOCK_INPUT();
    XFreePixmap(cur_Xscreen->display,stipples[idx]);
    UNBLOCK_INPUT();
    allocated_stipples[idx] = 0;
    return Qt;
  }
  else return Qnil;
}

Pixmap stipple_pixmap(stipple)
     Lisp_Object stipple;
{
  int idx;
  CHECK_NUMBER (stipple,0);
  idx = XFASTINT (stipple);
  if (idx < 0 || idx >= MAX_STIPPLES)
    error ("stipple id out of range: %u",idx);
  if (allocated_stipples[idx] == 0)
    error ("bad stipple id: %u",idx);
  return (stipples[idx]);
}

syms_of_stipple()
{
  defsubr(&Smake_stipple);
  defsubr(&Skill_stipple);
}
#endif /* BUTTON */
